function [XN,factors]=normalize(X,method,ppm,features)

% [XN,factors]=normalize(X,method,features)
%
% Normalize spectral matrix X to total area, single feature, or integral of set of features. 
%
% Arguments:
% 
% X               N x P matrix of spectral features for each sample
% method          'total' for Total Area, 'PQN' for Probablistic Quotient,
%                 'intensity' for normalization to single feature,
%                 'integral' for normalization to sum of set of features
% features        only required for 'intensity' or 'integral'.  For
%                 'intensity', the index of the feature in X to normalize
%                 to - e.g. [10] for X(:,10).  For 'integral', the range of
%                 features in X that span the peak to normalize to - eg
%                 [10,50] for X(:,10:50)
%
% Outputs:
% XN              N x P matrix of normalized data
% factors         N calculated normalization factors: XN = X / factors

p = inputParser;
addRequired(p,'X',@(x)validateattributes(x,{'numeric'}, {'2d'}));
addRequired(p,'method',@(x) any(validatestring(x,{'total','PQN','intensity','integral'})));

addOptional(p, 'ppm', [], @(x)validateattributes(x,{'numeric'}, {'row'}));
addOptional(p, 'features', [], @(x)validateattributes(x,{'numeric'}, {'scalar'}));

if exist('ppm','var')~=1
    ppm=0;
end
if exist('features','var')~=1
    features=0;
end

parse(p,X,method,ppm,features);

cellfun(@(f) evalin('caller',[f ' = p.Results.' f ';']), fieldnames(p.Results))

XN=zeros(size(X));

% if nargin<2
%     error('incorrect number of input parameters');
% end

[obs dim]=size(X);

switch method
    case 'total'
        factors=repmat(NaN,[1 obs]);
        for i=1:obs
            factors(i)=sum(X(i,:));
            XN(i,:)=X(i,:)./factors(i);
        end
    case 'PQN'
        X(0==X)=0.00000001;
        if nargin<4
            normRef=repmat(median(X),size(X,1),1);
        end
        F=X./(normRef);
        for i=1:obs
            XN(i,:)=X(i,:)./median(F(i,:));
            factors(i)=median(F(i,:));
        end
    case 'intensity'
        [~,feature]=min(abs(ppm-features));
        factors=X(:,feature);
        for i=1:obs
            XN(i,:)=X(i,:)./factors(i);
        end
    case 'integral'
        [~,feature(1)]=min(abs(ppm-features(1)));
        [~,feature(2)]=min(abs(ppm-features(2)));
        for i=1:obs
            factors(i)=trapz(X(i,feature(1):feature(2)));
            XN(i,:)=X(i,:)./factors(i);
        end
        
end
end