function XS=scale(X,method,offset)

% XS=scale(X,method,offset)
% Look out for heteroscedasticity when choosing a scaling method: 
%       -A lack of homogeneity in variance between variables.
%
% Arguments:
%
% X               N x P matrix of spectral features for each sample 
% method          'log' for log2 fold-change vs. median scaling, 'logoff' 
%                 for offset log2 fold-change vs. median,
%                 'mc' for mean-centering (only), 'auto' for autoscaling
%                 (mean-center and univariance scaling), 'pareto' for
%                 pareto scaling. 'integral' for normalization to sum of
%                 set of features
% offset          optional feature for 'log' or 'logoff'.  'log' uses
%                 default offset of 0.0001 to prevent InF, 'logoff' uses automatically
%                 calculated median.
%
% Outputs: 
% XS              N x P matrix of scaled data

switch method
    case 'mc'
        %Focus on the differences and not the similarities in the data
        %Advantage: Remove the offset from the data
        %Disadvantage: When data is heteroscedastic, the effect of this pretreatment method is not always sufficient
        XS=X-repmat(mean(X),[size(X,1),1]);
        
    case 'auto'
        %Compare metabolites based on correlations
        %Advantage: All metabolites become equally important, recommended
        %for peakpicked or binned data
        %Disadvantage: Inflation of the measurement errors, recommend not
        %using this if there is alot of noise in your spectra.
        XMC=X-repmat(mean(X),[size(X,1),1]);
        XS=XMC./repmat(std(X),[size(X,1),1]);
        
    case 'range'
        %Compare metabolites relative to the biological response range
        %Advantage: All metabolites become equally important.Scaling is related to biology 
        %Disadvantage: Inflation of the measurement errors and sensitive to outliers
        XMC=X-repmat(mean(X),[size(X,1),1]);
        Xmin=repmat(min(X,[],1),[size(X,1),1]);
        Xmax=repmat(max(X,[],1),[size(X,1),1]);
        XS=XMC./(Xmax-Xmin);
        
    case 'pareto'
        %Reduce the relative importance of large values, but keep data structure partially intact
        %Advantage: Stays closer to the original measurement than autoscaling
        %Disadvantage: Sensitive to large fold changes
        XMC=X-repmat(mean(X),[size(X,1),1]);
        XS=XMC./repmat(sqrt(std(X)),[size(X,1),1]);
        
    case 'vast'
        %Focus on the metabolites that show small fluctuations
        %Aims for robustness, can use prior group knowledge
        %Not suited for large induced variationwithout group structure
        XMC=X-repmat(mean(X),[size(X,1),1]);
        XS=XMC./repmat((std(X)),[size(X,1),1]);
        XA=repmat(mean(X),[size(X,1),1])./repmat((std(X)),[size(X,1),1]);
        XS=XS.*XA;
        
    case 'level'
        %Focus on relative response
        %Suited for identification of e.g. biomarkers
        %Inflation of the measurement errors
        XMC=X-repmat(mean(X),[size(X,1),1]);
        XS=XMC./repmat(mean(X),[size(X,1),1]);
        
%% Log Transformations: 
%Correct for heteroscedasticity, pseudo scaling. Make multiplicative models additive
%Advantage: Reduce heteroscedasticity, multiplicative effects become additive
%Disadvantage: Difficulties with values with large relative standard deviation and zeros

    case 'log'
        X=abs(X);
        if exist('offset')~=1
            offset=0.0001;
        end
        median_X=median(X+offset);
        FC=zeros(size(X));
        for k=1:size(X,1)
            FC(k,:)=[X(k,:)+offset]./median_X;
        end
                XS=log2(FC);
%         XS=log(FC);
    case 'logoff'
        X=abs(X);
        if exist('offset')~=1
            offset=median(X(X>0));
        end
        median_X=median(X+offset);
        FC=zeros(size(X));
        for k=1:size(X,1)
            FC(k,:)=[X(k,:)+offset]./median_X;
        end
        %         XS=log2(FC);
        XS=log(FC);
        
    case 'log10'
        X=abs(X);
       XMC=log10(X);
       XS=XMC-(repmat(mean(X),[size(X,1),1]));
       
    case 'log2'
        X=abs(X);
        XMC=log2(X);
        XS=XMC-(repmat(mean(X),[size(X,1),1]));
        
    case 'nlog'
        X=abs(X);
        XMC=log(X);
        XS=XMC-(repmat(mean(X),[size(X,1),1]));
        
%% Power Transfomations:
%Correct for heteroscedasticity, pseudo scaling
%Advantage: Reduce heteroscedasticity, no problems with small values
%Disadvantage: Choice for square root is arbitrary.
    case 'power'
        X=abs(X);
        if exist('offset')~=1
            offset=0.0001;
        end
        median_X=median(X+offset);
        FC=zeros(size(X));
        for k=1:size(X,1)
            FC(k,:)=[X(k,:)+offset]./median_X;
        end
        XS=pow2(FC);
    case 'poweroff'
         X=abs(X);
        if exist('offset')~=1
            offset=median(X(X>0));
        end
        median_X=median(X+offset);
        FC=zeros(size(X));
        for k=1:size(X,1)
            FC(k,:)=[X(k,:)+offset]./median_X;
        end
        XS=pow2(FC);
    case 'pwr'
        X=abs(X);
        XMC=pow2(X);
        XS=XMC-(repmat(mean(X),[size(X,1),1]));
        
 %% Only us this if you have the Glog script
    case 'optim_glog'
        X=abs(X);
        [XS,~,~]=optimise_glog_parameter(X);
        XS=XS';
    case 'glog'
        X=abs(X);
        [XS,~,~]=glog_opt(X);

end
end